@@ -1,4 +1,7 @@
 package System::Command::Reaper;
+{
+  $System::Command::Reaper::VERSION = '1.106';
+}
 
 use strict;
 use warnings;
@@ -7,11 +10,12 @@ use 5.006;
 use Carp;
 use Scalar::Util qw( weaken );
 
+use POSIX ":sys_wait_h";
+
+use constant MSWin32 => $^O eq 'MSWin32';
 use constant HANDLES => qw( stdin stdout stderr );
 use constant STATUS  => qw( exit signal core );
 
-our $VERSION = '1.01';
-
 sub new {
     my ($class, $command) = @_;
     my $self = bless { command => $command }, $class;
@@ -23,7 +27,50 @@ sub new {
     return $self;
 }
 
-sub reap {
+# this is necessary, because kill(0,pid) is misimplemented in perl core
+my $_is_alive = MSWin32
+    ? sub { return `tasklist /FO CSV /NH /fi "PID eq $_[0]"` =~ /^"/ }
+    : sub { return kill 0, $_[0]; };
+
+sub is_terminated {
+    my ($self) = @_;
+    my $pid = $self->{pid};
+
+    # Zed's dead, baby. Zed's dead.
+    return $pid if !$_is_alive->($pid) and exists $self->{command}{exit};
+
+    # If that is a re-animated body, we're gonna have to kill it.
+    return $self->_reap(WNOHANG);
+}
+
+sub _reap {
+    my ( $self, @flags ) = @_;
+    my $pid = $self->{pid};
+
+    if ( my $reaped = waitpid( $pid, @flags )
+        and !exists $self->{command}{exit} )
+    {
+        my $zed = $reaped == $pid;
+        carp "Child process already reaped, check for a SIGCHLD handler"
+            if !$zed && !$System::Command::QUIET;
+
+        @{$self}{ STATUS() }
+            = $zed
+            ? ( $? >> 8, $? & 127, $? & 128 )
+            : ( -1, -1, -1 );
+
+        # does our creator still exist?
+        @{ $self->{command} }{ STATUS() } = @{$self}{ STATUS() }
+            if defined $self->{command};
+
+        return $reaped;    # It's dead, Jim!
+    }
+
+    # Look! It's moving. It's alive. It's alive...
+    return;
+}
+
+sub close {
     my ($self) = @_;
 
     # close all pipes
@@ -33,37 +80,29 @@ sub reap {
     $err and $err->opened and $err->close || carp "error closing stderr: $!";
 
     # and wait for the child (if any)
-    if ( my $reaped = waitpid( $self->{pid}, 0 ) and !exists $self->{exit} ) {
-        my $zed = $reaped == $self->{pid};
-        carp "Child process already reaped, check for a SIGCHLD handler"
-            if !$zed && !$System::Command::QUIET;
-
-        # check $?
-        @{$self}{ STATUS() }
-            = $zed
-            ? ( $? >> 8, $? & 127, $? & 128 )
-            : ( -1, -1, -1 );
-
-        # does our creator still exist?
-        @{ $self->{command} }{ STATUS() } = @{$self}{ STATUS() }
-            if defined $self->{command};
-    }
+    $self->_reap();
 
     return $self;
 }
 
 sub DESTROY {
     my ($self) = @_;
-    $self->reap if !exists $self->{exit};
+    $self->close if !exists $self->{exit};
 }
 
 1;
 
-__END__
+
+
+=pod
 
 =head1 NAME
 
-System::Command::Reaper - Reap processes started by System::Command
+System::Command::Reaper
+
+=head1 VERSION
+
+version 1.106
 
 =head1 SYNOPSIS
 
@@ -72,9 +111,9 @@ Move along, nothing to see here.
 
 =head1 DESCRIPTION
 
-The C<System::Command> objects delegate the reaping of child
-processes to C<System::Command::Reaper> objects. This allows a user
-to create a C<System::Command> and discard it after having obtained
+The L<System::Command> objects delegate the reaping of child
+processes to System::Command::Reaper objects. This allows a user
+to create a L<System::Command> and discard it after having obtained
 one or more references to its handles connected to the child process.
 
 The typical use case looks like this:
@@ -100,27 +139,37 @@ Legend:
     | normal ref
     ! weak ref
 
-The C<System::Command::Reaper> object acts as a sentinel, that takes
-care of reaping the child process when the original C<System::Command>
-and its filehandles have been destroyed (or when C<System::Command>
+The System::Command::Reaper object acts as a sentinel, that takes
+care of reaping the child process when the original L<System::Command>
+and its filehandles have been destroyed (or when L<System::Command>
 C<close()> method is being called).
 
+=head1 NAME
+
+System::Command::Reaper - Reap processes started by System::Command
+
 =head1 METHODS
 
-C<System::Command::Reaper> supports the following methods:
+System::Command::Reaper supports the following methods:
 
 =head2 new( $command )
 
-Create a new C<System::Command::Reaper> object attached to the
-C<System::Command> object passed as a parameter.
+Create a new System::Command::Reaper object attached to the
+L<System::Command> object passed as a parameter.
 
-=head2 reap()
+=head2 close()
 
-Close all the opened filehandles of the main C<System::Command> object,
+Close all the opened filehandles of the main L<System::Command> object,
 reaps the child process, and updates the main object with the status
 information of the child process.
 
-C<DESTROY> calls C<reap()> when the sentinel is being destroyed.
+C<DESTROY> calls C<close()> when the sentinel is being destroyed.
+
+=head2 is_terminated()
+
+Returns a true value if the underlying process was terminated.
+
+If the process was indeed terminated, collects exit status, etc.
 
 =head1 AUTHOR
 
@@ -132,10 +181,9 @@ This scheme owes a lot to Vincent Pit who on #perlfr provided the
 general idea (use a proxy to delay object destruction and child process
 reaping) with code examples, which I then adapted to my needs.
 
-
 =head1 COPYRIGHT
 
-Copyright 2010-2011 Philippe Bruhat (BooK), all rights reserved.
+Copyright 2010-2013 Philippe Bruhat (BooK), all rights reserved.
 
 =head1 LICENSE
 
@@ -144,3 +192,6 @@ under the same terms as Perl itself.
 
 =cut
 
+
+__END__
+
